"""
Test the extends directive

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.forms.extends')

We can look for the extended form, it will contains fields and action
of the original one:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.base.ftests.forms.extends import Context
  >>> context = Context()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='othernameform')
  >>> form
  <zeam.form.base.ftests.forms.extends.OtherNameForm object at ...>

  >>> len(form.fields)
  1
  >>> list(form.fields)
  [<Field Name>]

  >>> len(form.actions)
  2
  >>> list(form.actions)
  [<DecoratedAction Register>, <DecoratedAction Kill>]



"""

from zeam.form import base
from grokcore import component as grok


class Context(grok.Context):
    pass


class NameForm(base.Form):

    label = u"Name"
    description = u"Name form"
    fields = base.Fields(base.Field("Name"))
    fields['name'].description = 'Name of the candidate'
    fields['name'].required = True

    @base.action(u"Register")
    def register(self):
        data, errors = self.extractData()
        if errors:
            return
        # In case of success we don't keep request value in the form
        self.ignoreRequest = True
        self.status = u"Registered %(name)s" % data


class OtherNameForm(NameForm):
    base.extends(NameForm)

    @base.action(u"Kill")
    def kill(self):
        data, errors = self.extractData()
        if errors:
            return
        self.status = u"Dead man you are %(name)s" % data



"""
We are going to define a custom widget here which define its rendering
HTML using a template associated with Grok.

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.widgets.widgettemplate')

So now should be to lookup our widget:

  >>> from zeam.form.base.ftests.widgets.widgettemplate import MyField
  >>> field = MyField("Cool Template Test")
  >>> field
  <MyField Cool Template Test>

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.base.form import Form
  >>> form = Form(None, request)

  >>> from zeam.form.base import interfaces
  >>> from zope import component
  >>> widget = component.getMultiAdapter(
  ...     (field, form, request), interfaces.IWidget)
  >>> widget
  <MyWidget Cool Template Test>

And we are able now to call its render method:

  >>> print widget.render()
  <p>Nice template for Cool Template Test</p>


"""

from zeam.form.base.fields import Field
from zeam.form.base.widgets import Widget
from zeam.form.base import interfaces
from zope.interface import Interface
from grokcore import component as grok


class MyField(Field):
    """A custom field.
    """


class MyWidget(Widget):
    """Custom widget to render my field.
    """
    grok.adapts(MyField, interfaces.IFormCanvas, Interface)



# -*- coding: utf-8 -*-

import martian
import grokcore.view
import grokcore.component
import grokcore.security

from grokcore.security.util import protect_getattr
from grokcore.view.meta.views import TemplateGrokker

from zeam.form.base.widgets import Widget
from zeam.form.base.form import StandaloneForm, GrokViewSupport

from zope import interface, component
from zope.publisher.interfaces.browser import IBrowserPage
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class WidgetTemplateGrokker(TemplateGrokker):
    martian.component(Widget)

    def has_render(self, factory):
        return factory.render != Widget.render

    def has_no_render(self, factory):
        return not self.has_render(factory)


class FormTemplateGrokker(TemplateGrokker):
    martian.component(GrokViewSupport)

    def has_render(self, factory):
        return factory.render != GrokViewSupport.render

    def has_no_render(self, factory):
        return False


class FormGrokker(grokcore.view.meta.views.ViewGrokker):
    martian.component(StandaloneForm)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.component.name,
                      get_default=grokcore.view.meta.views.default_view_name)

    def execute(self, factory, config, context, layer, name, **kw):
        # __view_name__ is needed to support IAbsoluteURL on views
        factory.__view_name__ = name
        adapts = (context, layer)

        config.action(
            discriminator=('adapter', adapts, interface.Interface, name),
            callable=component.provideAdapter,
            args=(factory, adapts, interface.Interface, name))
        return True


class FormSecurityGrokker(martian.ClassGrokker):
    martian.component(StandaloneForm)
    martian.directive(grokcore.security.require, name='permission')

    def execute(self, factory, config, permission, **kw):
        # XXX use something else than IBrowserPage ? (trollfot)
        for method_name in IBrowserPage:
            config.action(
                discriminator=('protectName', factory, method_name),
                callable=protect_getattr,
                args=(factory, method_name, permission))
        return True



# -*- coding: utf-8 -*-

import operator
import sys

from grokcore import component as grok
from grokcore.view import util

from zeam.form.base import interfaces
from zeam.form.base.actions import Actions
from zeam.form.base.datamanager import ObjectDataManager
from zeam.form.base.errors import Errors, Error
from zeam.form.base.fields import Fields
from zeam.form.base.markers import NO_VALUE, INPUT
from zeam.form.base.widgets import Widgets, getWidgetExtractor
from zeam.form.base.interfaces import ICollection, IError

from zope import component, i18n, interface
from zope.pagetemplate.interfaces import IPageTemplate
from zope.publisher.browser import BrowserPage
from zope.publisher.publish import mapply
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('zeam.form.base')


class Object(object):
    """Python object that takes argument to its __init__, in order to
    use super. This is required by Python 2.6.
    """

    def __init__(self, *args, **kwargs):
        pass


class GrokViewSupport(Object):
    """Support Grok view like behavior, without inheriting of Grok
    view (not to get any grokker at all, or inherit from BrowerView,
    BrowserPage).

    The render method support IPageTemplate in addition to Grok template.
    """
    grok.baseclass()
    grok.implements(interfaces.IGrokViewSupport)

    i18nLanguage = None

    def __init__(self, context, request):
        super(GrokViewSupport, self).__init__(context, request)
        self.context = context
        self.request = request

        if getattr(self, 'module_info', None) is not None:
            self.static = component.queryAdapter(
                self.request,
                interface.Interface,
                name=self.module_info.package_dotted_name)
            if self.static is not None:
                # For security in Zope 2
                self.static.__parent__ = self
        else:
            self.static = None

    @property
    def response(self):
        return self.request.response

    def redirect(self, url):
        self.response.redirect(url)

    def url(self, obj=None, name=None, data=None):
        """Return string for the URL based on the obj and name. The data
        argument is used to form a CGI query string.
        """
        if isinstance(obj, basestring):
            if name is not None:
                raise TypeError(
                    'url() takes either obj argument, obj, string arguments, '
                    'or string argument')
            name = obj
            obj = None

        if obj is None:
            obj = self.context
        if data is None:
            data = {}
        else:
            if not isinstance(data, dict):
                raise TypeError('url() data argument must be a dict.')

        return util.url(self.request, obj, name, data=data)

    def default_namespace(self):
        namespace = {'view': self,
                     'context': self.context,
                     'request': self.request}
        if self.i18nLanguage is not None:
            namespace['target_language'] = self.i18nLanguage
        return namespace

    def namespace(self):
        return {}

    def update(self):
        pass

    def render(self):
        # Try grok template first
        template = getattr(self, 'template', None)
        if template is not None:
            return self.template.render(self)
        # Fallback on IPageTemplate
        template = component.getMultiAdapter(
            (self, self.request), IPageTemplate)
        return template()


_marker = object()


def cloneFormData(original, content=_marker, prefix=None):
    assert interfaces.IFieldExtractionValueSetting.providedBy(original)
    clone = FormData(original.context, original.request, content)
    clone.ignoreRequest = original.ignoreRequest
    clone.ignoreContent = original.ignoreContent
    clone.mode = original.mode
    clone.parent = original
    if prefix is None:
        clone.prefix = original.prefix
    else:
        clone.prefix = prefix
    # XXX Those fields are not checked by the interface
    clone.i18nLanguage = original.i18nLanguage
    clone.postOnly = original.postOnly
    errors = original.errors.get(clone.prefix, None)
    if errors is not None:
        clone.errors = errors
    return clone


class FieldsValues(dict):
    """Dictionary to contains values of fields. get default by default
    on the default value of a field.
    """

    def __init__(self, form, fields):
        self.form = form
        self.fields = fields

    def getWithDefault(self, key, default=None):
        value = super(FieldsValues, self).get(key, default)
        if value is NO_VALUE:
            value = self.fields[key].getDefaultValue(self.form)
            if value is NO_VALUE:
                return default
        return value

    def getDictWithDefault(self, default=None):
        result = {}
        for key in self.keys():
            result[key] = self.getWithDefault(key, default=default)
        return result

    # BBB
    getDefault = getWithDefault


class FormData(Object):
    """This represent a submission of a form. It can be used to update
    widgets and run actions.
    """
    grok.baseclass()
    grok.implements(interfaces.IFormData)

    prefix = 'form'
    parent = None
    mode = INPUT
    dataManager = ObjectDataManager
    dataValidators = []
    postOnly = True
    i18nLanguage = None

    ignoreRequest = False
    ignoreContent = True

    status = u''

    def __init__(self, context, request, content=_marker):
        super(FormData, self).__init__(context, request)
        self.context = context
        self.request = request
        self.errors = Errors()
        self.__extracted = {}
        self.__content = None
        if content is _marker:
            content = context
        self.setContentData(content)

    @property
    def formErrors(self):
        error = self.errors.get(self.prefix, None)
        if error is None:
            return []
        if ICollection.providedBy(error):
            return error
        return [error]

    def htmlId(self):
        return self.prefix.replace('.', '-')

    def getContent(self):
        # Shortcut for actions. You should not reimplement that method
        # but getContentData.
        return self.getContentData().getContent()

    def getContentData(self):
        return self.__content

    def setContentData(self, content):
        if not interfaces.IDataManager.providedBy(content):
            content = self.dataManager(content)
        self.__content = content

    def validateData(self, fields, data, errors):
        for factory in self.dataValidators:
            validator = factory(self, fields)
            for error in validator.validate(data):
                if not IError.providedBy(error):
                    error = Error(error, self.prefix)
                errors.append(error)
        if len(errors):
            if self.prefix not in errors:
                errors.append(Error(_(u"There were errors."), self.prefix))
        return errors

    def extractData(self, fields):
        # XXX to review this
        cached = self.__extracted.get(fields)
        if cached is not None:
            return cached
        data = FieldsValues(self, fields)
        errors = Errors()
        self.__extracted[fields] = (data, errors)

        for field in fields:
            if not field.available(self):
                continue

            # Widget extraction and validation
            extractor = getWidgetExtractor(field, self, self.request)
            if extractor is not None:
                value, error = extractor.extract()
                if error is None:
                    error = field.validate(value, self)
                if error is not None:
                    if not IError.providedBy(error):
                        error = Error(error, extractor.identifier)
                    errors.append(error)
                data[field.identifier] = value

        # Generic form validation
        errors = self.validateData(fields, data, errors)
        self.errors = errors
        return (data, errors)


class FormCanvas(GrokViewSupport, FormData):
    """This represent a sumple form setup: setup some fields and
    actions, prepare widgets for it.
    """
    grok.baseclass()
    grok.implements(interfaces.ISimpleFormCanvas)

    label = u''
    description = u''

    actions = Actions()
    fields = Fields()

    def __init__(self, context, request):
        super(FormCanvas, self).__init__(context, request)
        self.actionWidgets = Widgets(form=self, request=self.request)
        self.fieldWidgets = Widgets(form=self, request=self.request)

    def extractData(self, fields=None):
        if fields is None:
            fields = self.fields
        return super(FormCanvas, self).extractData(fields)

    def haveRequiredFields(self):
        return reduce(
            operator.or_,
            [False] + map(operator.attrgetter('required'), self.fields))

    def updateActions(self):
        return self.actions.process(self, self.request)

    def updateWidgets(self):
        self.fieldWidgets.extend(self.fields)
        self.actionWidgets.extend(self.actions)

        self.fieldWidgets.update()
        self.actionWidgets.update()


class StandaloneForm(GrokViewSupport, BrowserPage):
    """This is a base for a standalone form, process the form.
    """
    grok.baseclass()

    def updateActions(self):
        return self, None, None

    def updateWidgets(self):
        pass

    def updateForm(self):
        self.updateActions()
        self.updateWidgets()

    def __call__(self):
        mapply(self.update, (), self.request)
        if self.response.getStatus() in (302, 303):
            # A redirect was triggered somewhere in update().  Don't
            # continue processing the form
            return

        if self.i18nLanguage is None:
            self.i18nLanguage = i18n.negotiate(self.request)
        self.updateForm()
        if self.response.getStatus() in (302, 303):
            return

        return self.render()


class Form(FormCanvas, StandaloneForm):
    """A full simple standalone form.
    """
    grok.baseclass()
    grok.implements(interfaces.ISimpleForm)


def extends(*forms, **opts):
    # Extend a class with parents components
    field_type = opts.get('fields', 'all')

    def extendComponent(field_type):
        factory = {'actions': Actions, 'fields': Fields}.get(field_type)
        if factory is None:
            raise ValueError(u"Invalid parameter fields to extends")
        frame = sys._getframe(2)
        f_locals = frame.f_locals
        components = f_locals.setdefault(field_type, factory())
        components.extend(*map(operator.attrgetter(field_type), forms))

    if field_type == 'all':
        extendComponent('actions')
        extendComponent('fields')
    else:
        extendComponent(field_type)



# -*- coding: utf-8 -*-

from grokcore import view as grok
from megrok import pagetemplate as pt
from zeam.form.base import form

grok.templatedir('default_templates')


class FormTemplate(pt.PageTemplate):
    pt.view(form.Form)



# -*- coding: utf-8 -*-


def sort_components(order):
    if not isinstance(order, list) and not isinstance(order, tuple):
        raise ValueError(
            "Please provide a valid list or tuple of component identifiers.")
    if len(order) < 2:
        raise ValueError(
            "Please provide a list of, at least, two component identifiers.")

    def compare(c1, c2, order=order):

        try:
            o1 = order.index(c1)
        except ValueError:
            return 1

        try:
            o2 = order.index(c2)
        except ValueError:
            return -1

        if o1 - o2 > 1:
            return 1
        if o1 - o2 < 1:
            return -1

        return 0

    return compare



# -*- coding: utf-8 -*-

from grokcore import component as grok
from zeam.form.base import interfaces
from zeam.form.base.components import Component, Collection
from zeam.form.base.interfaces import IModeMarker
from zeam.form.base.markers import NO_VALUE, getValue, HiddenMarker
from zope import component
from zope.interface import Interface
from zope.pagetemplate.interfaces import IPageTemplate


def widget_id(form, component):
    """Create an unique ID for a widget.
    """
    return '.'.join(
        (iditem for iditem in
         (str(form.prefix), component.prefix, component.identifier,)
         if iditem))


class Widget(Component, grok.MultiAdapter):
    grok.baseclass()
    grok.implements(interfaces.IWidget)
    grok.provides(interfaces.IWidget)

    def __init__(self, component, form, request):
        identifier = widget_id(form, component)
        super(Widget, self).__init__(component.title, identifier)
        self.component = component
        self.form = form
        self.request = request

    def copy(self):
        raise NotImplementedError

    def htmlId(self):
        # Return an identifier suitable for CSS usage
        return self.identifier.replace('.', '-')

    def htmlClass(self):
        return 'field'

    def isVisible(self):
        return not isinstance(self.component.mode, HiddenMarker)

    def default_namespace(self):
        namespace = {'widget': self,
                     'request': self.request}
        if self.form.i18nLanguage is not None:
            # i18n support for Chameleon
            namespace['target_language'] = self.form.i18nLanguage
        return namespace

    def namespace(self):
        return {}

    def update(self):
        pass

    def render(self):
        # Try grok template first
        template = getattr(self, 'template', None)
        if template is not None:
            return self.template.render(self)
        # Fallback on IPageTemplate
        template = component.getMultiAdapter(
            (self, self.request), IPageTemplate)
        return template()


class WidgetExtractor(grok.MultiAdapter):
    grok.implements(interfaces.IWidgetExtractor)
    grok.provides(interfaces.IWidgetExtractor)
    grok.adapts(
        interfaces.IRenderableComponent,
        interfaces.IFieldExtractionValueSetting,
        Interface)

    def __init__(self, component, form, request):
        self.identifier = widget_id(form, component)
        self.component = component
        self.form = form
        self.request = request

    def extract(self):
        value = self.request.form.get(self.identifier, u'')
        if not len(value):
            value = NO_VALUE
        return (value, None)

    def extractRaw(self):
        entries = {}
        sub_identifier = self.identifier + '.'
        for key, value in self.request.form.iteritems():
            if key.startswith(sub_identifier) or key == self.identifier:
                entries[key] = value
        return entries


class HiddenWidgetExtractor(WidgetExtractor):
    grok.name('hidden')


class ReadOnlyWidgetExtractor(WidgetExtractor):
    grok.name('readonly')


def createWidget(field, form, request):
    """Create a widget (or return None) for the given form and
    request.
    """
    if not field.available(form):
        return None
    mode = str(getValue(field, 'mode', form))
    return component.getMultiAdapter(
        (field, form, request), interfaces.IWidget, name=mode)


class Widgets(Collection):
    grok.implements(interfaces.IWidgets)

    type = interfaces.IWidget

    def extend(self, *args):
        if not args:
            return

        # Ensure the user created us with the right options
        assert self.__dict__.get('form', None) is not None
        assert self.__dict__.get('request', None) is not None

        for arg in args:
            if interfaces.IWidgets.providedBy(arg):
                for widget in arg:
                    self.append(widget)
            elif interfaces.IIterable.providedBy(arg):
                for field in arg:
                    widget = createWidget(field, self.form, self.request)
                    if widget is not None:
                        self.append(widget)
            elif interfaces.IWidget.providedBy(arg):
                self.append(arg)
            elif interfaces.IRenderableComponent.providedBy(arg):
                widget = createWidget(arg, self.form, self.request)
                if widget is not None:
                    self.append(widget)
            else:
                raise TypeError(u'Invalid type', arg)

    def update(self):
        for widget in self:
            widget.update()


# After follow the implementation of some really generic default
# widgets

class ActionWidget(Widget):
    grok.adapts(
        interfaces.IAction,
        interfaces.IFieldExtractionValueSetting,
        Interface)
    grok.name('input')

    def __init__(self, component, form, request):
        super(ActionWidget, self).__init__(component, form, request)
        self.description = component.description
        self.accesskey = component.accesskey

    def htmlClass(self):
        return 'action'


def getWidgetExtractor(field, form, request):
    mode = str(getValue(field, 'mode', form))

    # The field mode should be extractable or we skip it.
    if (IModeMarker.providedBy(field.mode) and
        field.mode.extractable is False):
        return None

    extractor = component.queryMultiAdapter(
        (field, form, request), interfaces.IWidgetExtractor, name=mode)
    if extractor is not None:
        return extractor
    return component.getMultiAdapter(
        (field, form, request), interfaces.IWidgetExtractor)


class FieldWidget(Widget):
    grok.implements(interfaces.IFieldWidget)
    grok.adapts(
        interfaces.IField,
        interfaces.IFormData,
        Interface)
    grok.name('input')

    def __init__(self, component, form, request):
        super(FieldWidget, self).__init__(component, form, request)
        self.description = component.description
        self.required = component.isRequired(form)
        self.readonly = component.readonly

    @property
    def error(self):
        return self.form.errors.get(self.identifier, None)

    def computeValue(self):
        # First lookup the request
        ignoreRequest = getValue(self.component, 'ignoreRequest', self.form)
        if not ignoreRequest:
            extractor = getWidgetExtractor(
                self.component, self.form, self.request)
            if extractor is not None:
                value = extractor.extractRaw()
                if value:
                    return self.prepareRequestValue(value, extractor)

        # After, the context
        ignoreContent = getValue(self.component, 'ignoreContent', self.form)
        if not ignoreContent:
            data = self.form.getContentData()
            try:
                value = data.get(self.component.identifier)
                # XXX: Need review
                if value is None:
                    value = NO_VALUE
                return self.prepareContentValue(value)
            except KeyError:
                # No value on the content for field, continue.
                pass

        # Take any default value
        value = self.component.getDefaultValue(self.form)
        return self.prepareContentValue(value)

    def valueToUnicode(self, value):
        return unicode(value)

    def prepareRequestValue(self, value, extractor):
        return value

    def prepareContentValue(self, value):
        formatted_value = u''
        if value is not NO_VALUE:
            formatted_value = self.valueToUnicode(value)
        return {self.identifier: formatted_value}

    def inputValue(self, id=None):
        if id is not None:
            id = '%s.%s' % (self.identifier, id)
        else:
            id = self.identifier
        return self.value.get(id, '')

    def update(self):
        self.value = self.computeValue()


class DisplayFieldWidget(FieldWidget):
    grok.name('display')


class HiddenFieldWidget(FieldWidget):
    grok.name('hidden')


class ReadOnlyFieldWidget(FieldWidget):
    grok.name('readonly')



# -*- coding: utf-8 -*-

from zeam.form.base import interfaces
from zeam.form.base.components import Component, Collection, _marker
from zope.interface import implements, directlyProvides
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('zeam.form.base')


class Error(Component):
    implements(interfaces.IError)

    def get(self, prefix, default=_marker):
        # We implements get to be compatible with the sub-error protocol.
        if self.identifier == prefix:
            return self
        if default is _marker:
            raise KeyError(prefix)
        return default


class Errors(Collection):
    implements(interfaces.IErrors)

    type = interfaces.IError

    def __init__(self, *components, **options):
        Collection.__init__(self, *components, **options)
        if 'identifier' in options:
            directlyProvides(self, interfaces.IError)

    @property
    def title(self):
        if interfaces.IError.providedBy(self):
            default_error = self.get(self.identifier, None)
            if default_error is not None:
                return default_error.title
            return _(u"There were errors.")
        raise AttributeError('property')

    def append(self, component):
        if self.type.providedBy(component):
            if component.identifier in self:
                previous = self[component.identifier]
                if not interfaces.IErrors.providedBy(previous):
                    previous.identifier += '.0'
                    collection = self.__class__(
                        previous, identifier=component.identifier)
                    self[component.identifier] = collection
                else:
                    collection = previous
                component.identifier += '.%d' % len(collection)
                collection.append(component)
            else:
                super(Errors, self).append(component)
        else:
            raise TypeError(u"Invalid type", component)

    def clone(self, new_identifier=None):
        raise NotImplementedError(u'Errors collections are not clonable.')

    def __repr__(self):
        if interfaces.IError.providedBy(self):
            return "<%s for %r>" % (self.__class__.__name__, self.identifier)
        return Collection.__repr__(self)




import unittest
from pkg_resources import resource_listdir
from zope.testing import doctest
from zeam.form.base.testing import FunctionalLayer


def suiteFromPackage(name):
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    globs = {'getRootFolder': FunctionalLayer.getRootFolder}
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'zeam.form.base.ftests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(dottedname,
                                    extraglobs=globs,
                                    optionflags=optionflags)
        test.layer = FunctionalLayer
        suite.addTest(test)
    return suite


def test_suite():
    suite = unittest.TestSuite()
    for name in ['forms', 'widgets']:
        suite.addTest(suiteFromPackage(name))
    return suite




from zeam.form.base.interfaces import IDataManager
from zope.interface import implements


class BaseDataManager(object):
    """Base class for a data manager.
    """
    implements(IDataManager)

    def __init__(self, content):
        self.content = content

    def getContent(self):
        return self.content

    def get(self, identifier):
        raise NotImplementedError

    def set(self, identifier, value):
        raise NotImplementedError

    def __repr__(self):
        return '<%s used for %r>' % (self.__class__.__name__, self.content)


class ObjectDataManager(BaseDataManager):
    """An object data manager, which look up data as attributes on
    objects.
    """

    def get(self, identifier):
        try:
            return getattr(self.content, identifier)
        except AttributeError:
            raise KeyError(identifier)

    def set(self, identifier, value):
        setattr(self.content, identifier, value)


class DictDataManager(BaseDataManager):
    """A dictionary data manager, which look up data as keys in the
    dictionary.
    """

    def get(self, identifier):
        return self.content[identifier]

    def set(self, identifier, value):
        self.content[identifier] = value


class NoneDataManager(BaseDataManager):
    """A null data manager, which return directly the given object as
    data.
    """

    def get(self, identifier):
        return self.content


def makeAdaptiveDataManager(interface):

    class AdaptiveDataManager(ObjectDataManager):
        """A data manager that adapt its content to an interface
        before doing anything.
        """

        def __init__(self, content):
            self.content = interface(content)

    return AdaptiveDataManager




from zeam.form.base import interfaces
from zeam.form.base.components import Component, Collection
from zeam.form.base.markers import NO_VALUE, DEFAULT

from zope.interface import implements
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('zeam.form.base')


class Field(Component):
    implements(interfaces.IField)

    description = u''
    required = False
    prefix = 'field'
    readonly = False

    ignoreContent = DEFAULT
    ignoreRequest = DEFAULT
    mode = DEFAULT
    defaultValue = NO_VALUE

    def available(self, form):
        return True

    def isRequired(self, form):
        if callable(self.required):
            return self.required(form)
        return self.required

    def getDefaultValue(self, form):
        if callable(self.defaultValue):
            return self.defaultValue(form)
        return self.defaultValue

    def isEmpty(self, value):
        return value is NO_VALUE

    def validate(self, value, form):
        if self.isRequired(form) and self.isEmpty(value):
            return _(u"Missing required value.")
        return None


class Fields(Collection):
    implements(interfaces.IFields)

    type = interfaces.IField
    factory = interfaces.IFieldFactory



"""
We are going to define a simple form with an action.

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.forms.simpleform')

We can now lookup our form by the name of its class:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.base.ftests.forms.simpleform import Context
  >>> context = Context()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='change')
  >>> form
  <zeam.form.base.ftests.forms.simpleform.Change object at ...>

  >>> len(form.fields)
  0
  >>> len(form.actions)
  1

And we can render it:

  >>> print form()
  <html>
    <head>
    </head>
    <body>
      <form action="http://127.0.0.1" method="post"
            enctype="multipart/form-data">
        <h1>My form</h1>
        <p>The description of my form</p>
        <div class="actions">
           <div class="action">
              <input type="submit" id="form-action-change-me"
                     name="form.action.change-me" value="Change Me" class="action" />
           </div>
        </div>
      </form>
    </body>
  </html>


Integration tests
-----------------

Let's try to take a browser and submit that form:

  >>> root = getRootFolder()
  >>> root['test_content'] = context

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

  >>> browser.open('http://localhost/test_content/change')
  >>> action = browser.getControl('Change Me')
  >>> action
  <SubmitControl name='form.action.change-me' type='submit'>

  >>> action.click()

  >>> 'I completely changed everything' in browser.contents
  True

"""

from zeam.form import base
from grokcore import component as grok


class Context(grok.Context):
    pass


class ChangeAction(base.Action):

    def __call__(self, submission):
        submission.status = u"I completely changed everything"


class Change(base.Form):

    label = u"My form"
    description = u"The description of my form"
    actions = base.Actions(ChangeAction("Change Me"))



"""
We are going to define a custom widget here which define its rendering
HTML using a render method.

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.widgets.widgetrender')

So now should be to lookup our widget:

  >>> from zeam.form.base.ftests.widgets.widgetrender import MyField
  >>> field = MyField("Cool Test")
  >>> field
  <MyField Cool Test>

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.base.form import Form
  >>> form = Form(None, request)

  >>> from zeam.form.base import interfaces
  >>> from zope import component
  >>> widget = component.getMultiAdapter(
  ...     (field, form, request), interfaces.IWidget)
  >>> widget
  <MyWidget Cool Test>

And we are able now to call its render method:

  >>> print widget.render()
  <p>Too complicated widget for Cool Test</p>

"""

from zeam.form.base.fields import Field
from zeam.form.base.widgets import Widget
from zeam.form.base import interfaces
from zope.interface import Interface
from grokcore import component as grok


class MyField(Field):
    """A custom field.
    """


class MyWidget(Widget):
    """Custom widget to render my field
    """
    grok.adapts(MyField, interfaces.IFormCanvas, Interface)

    def render(self):
        return u"<p>Too complicated widget for %s</p>" % (
            self.component.title)



# -*- coding: utf-8 -*-

from zeam.form.base.interfaces import IModeMarker
from zope.interface import implements


class Marker(object):
    """Marker object, designed to be used as singleton (like None,
    True, False).
    """

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name.lower()

    def __repr__(self):
        return '<Marker %s>' % (self.name.upper())


class ModeMarker(Marker):
    """A Marker defining a form mode. It has a specific attribute,
    extractable, that defines if the mode allows the data extraction
    """
    implements(IModeMarker)

    def __init__(self, name, extractable=True):
        Marker.__init__(self, name)
        self.extractable = extractable


class HiddenMarker(ModeMarker):
    """A marker that hides a field.
    """
    pass


# Data extraction markers
SUCCESS = Marker('SUCCESS')
FAILURE = Marker('FAILURE')
DEFAULT = Marker('DEFAULT')
NO_VALUE = Marker('NO_VALUE')
NO_CHANGE = Marker('NO_CHANGE')
NOTHING_DONE = Marker('NOTHING_DONE')

# Mode markers
DISPLAY = ModeMarker('DISPLAY', extractable=False)
INPUT = ModeMarker('INPUT')
HIDDEN = HiddenMarker('HIDDEN')


def getValue(object, attr, default_object):
    """Retrieve the attr value from object. If it's DEFAULT, try to
    look up it on the default_object.
    """
    value = getattr(object, attr, DEFAULT)
    if value is DEFAULT:
        value = getattr(default_object, attr)
    return value



from setuptools import setup, find_packages
import os

version = '1.2'

tests_require = [
    'zope.app.wsgi',
    'grokcore.view [test]',
    ]

setup(name='zeam.form.base',
      version=version,
      description="Grok based form framework",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='grok form framework',
      author='Sylvain Viollon',
      author_email='thefunny@gmail.com',
      url='http://pypi.python.org/pypi/zeam.form.base',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['zeam', 'zeam.form'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'grokcore.component',
          'grokcore.security',
          'grokcore.view >= 2.2',
          'martian',
          'megrok.chameleon',
          'megrok.pagetemplate >= 0.7',
          'setuptools',
          'zope.component',
          'zope.i18n',
          'zope.interface',
          'zope.pagetemplate',
          'zope.publisher',
          'zope.testing',
          ],
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )




import sys
import itertools

from zeam.form.base import interfaces
from zeam.form.base.components import Component, Collection
from zeam.form.base.errors import Error
from zeam.form.base.markers import NO_VALUE, NOTHING_DONE, FAILURE
from zeam.form.base.markers import getValue, DEFAULT

from zope.interface import implements, alsoProvides
from zope import component


class Action(Component):
    """A form action.
    """
    implements(interfaces.IAction)

    prefix = 'action'
    # By default an action is always in input mode (there is not much
    # sense otherwise).
    mode = 'input'
    description = None
    accesskey = None
    postOnly = DEFAULT

    def available(self, form):
        return True

    def validate(self, form):
        return True

    def __call__(self, submission):
        raise NotImplementedError


class Actions(Collection):
    """A list of form action.
    """
    implements(interfaces.IActions)

    type = interfaces.IAction

    def process(self, form, request):
        for action in self:
            extractor = component.getMultiAdapter(
                (action, form, request), interfaces.IWidgetExtractor)

            value, error = extractor.extract()
            if value is not NO_VALUE:
                isPostOnly = getValue(action, 'postOnly', form)
                if isPostOnly and request.method != 'POST':
                    form.errors.append(
                        Error('This form was not submitted properly',
                              form.prefix))
                    return form, None, FAILURE
                try:
                    if action.validate(form):
                        return form, action, action(form)
                except interfaces.ActionError, error:
                    form.errors.append(Error(error.args[0], form.prefix))
                    return form, action, FAILURE
        return form, None, NOTHING_DONE


class CompoundActions(object):
    """Compound different types of actions together.
    """
    implements(interfaces.IIterable)

    def __init__(self, *new_actions):
        self.__actions = []
        self.extend(new_actions)

    def extend(self, new_actions):
        for actions in new_actions:
            self.append(actions)

    def append(self, actions):
        assert interfaces.IActions.providedBy(actions), u"Invalid actions"
        self.__actions.append(actions)

    def copy(self):
        copy = self.__class__()
        copy.extend(self.__actions)
        return copy

    def process(self, form, request):
        for actions in self.__actions:
            action, status = actions.process(form, request)
            if status != NOTHING_DONE:
                break
        return action, status

    def __add__(self, actions):
        copy = self.copy()
        copy.extend(actions)
        return copy

    def __iter__(self):
        return itertools.chain(*self.__actions)


# Convience API, decorator to add action

class DecoratedAction(Action):
    """An action created by a decorator.
    """

    def __init__(self, title, callback,
                 identifier=None, description=None, accesskey=None,
                 validator=None, available=None):
        super(Action, self).__init__(title, identifier)
        self._callback = callback
        self._validator = validator
        self._available = available
        self.accesskey = accesskey
        self.description = description

    def validate(self, form):
        if self._validator is not None:
            return self._validator(form)
        return True

    def available(self, form):
        if self._available is not None:
            return self._available(form)
        return True

    def __call__(self, form, *args, **kwargs):
        assert self._callback is not None
        return self._callback(form, *args, **kwargs)


# More convienent, extract the data before calling the action

class ExtractedDecoratedAction(DecoratedAction):

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return FAILURE
        # We directly give data.
        return super(ExtractedDecoratedAction, self).__call__(form, data)


def action(title, identifier=None, description=None, accesskey=None,
           validator=None, available=None, implements=None,
           factory=DecoratedAction, category='actions'):
    def createAction(callback):
        new_action = factory(
            title, callback, identifier, description, accesskey,
            validator, available)
        if implements is not None:
            alsoProvides(new_action, implements)

        # Magic to access the parent action list to add the action
        frame = sys._getframe(1)
        f_locals = frame.f_locals
        actions = f_locals.setdefault(category, Actions())

        actions.append(new_action)

        # We keep the same callback, so we can do super in
        # subclass. Registering it is enough, we do not need something
        # else.
        return callback
    return createAction



# -*- coding: utf-8 -*-

import re

from pkg_resources import iter_entry_points
from zope import component
from zope.interface import implements
from zope.testing import cleanup

from zeam.form.base import interfaces


_valid_identifier = re.compile('[A-Za-z][A-Za-z0-9_-]*$')


def createId(name):
    # Create a valid id from any string.
    identifier = unicode(name).strip().encode('utf-8').replace(' ', '-')
    if _valid_identifier.match(identifier):
        return identifier.lower()
    return identifier.encode('hex')


class Component(object):
    implements(interfaces.IComponent)

    identifier = None
    title = None

    def __init__(self, title=None, identifier=None):
        if not self.title:
            if not title:
                # If the title is empty, use the identifier as title
                title = identifier
                if title is None:
                    raise ValueError(
                        u"Need at least a title to build a component.")
            self.title = title
        if identifier is None:
            identifier = createId(self.title)
        self.identifier = str(identifier)

    def clone(self, new_identifier=None):
        return self.__class__(self.title, new_identifier)

    def __repr__(self):
        return "<%s %s>" % (self.__class__.__name__, self.title)


_marker = object()
_loaded = False


def loadComponents():
    """Goes through all available components loaders and call them.
    """
    global _loaded
    if _loaded:
        return
    for loader_entry in iter_entry_points('zeam.form.components'):
        loader = loader_entry.load()
        if not callable(loader):
            raise TypeError(
                'Entry point %r should be a callable to register  components'
                % loader_entry.name)
        loader()
    _loaded = True


def reloadComponents():
    """Reload all zeam components.

    This is mainly used by testing layers.
    """
    global _loaded
    _loaded = False
    loadComponents()


cleanup.addCleanUp(reloadComponents)


class Collection(object):
    """Represent a collection of components.
    """
    implements(interfaces.ICollection)

    type = interfaces.IComponent
    factory = None

    def __init__(self, *components, **options):
        self.__options = {}
        for name, value in options.items():
            if name not in interfaces.ICollection:
                self.__options[name] = value
        self.__dict__.update(self.__options)
        self.__ids = []
        self.__components = []
        if len(components):
            self.extend(*components)

    def reverse(self):
        self.__components = [c for c in reversed(self.__components)]
        self.__ids = [c.identifier for c in self.__components]

    def sort(self, cmp=cmp, key=lambda c: c.identifier, reverse=False):
        self.__components.sort(cmp=cmp, key=key, reverse=reverse)
        self.__ids = [c.identifier for c in self.__components]

    def clear(self):
        self.__ids = []
        self.__components = []

    def get(self, id, default=_marker):
        try:
            return self.__components[self.__ids.index(id)]
        except ValueError:
            if default is _marker:
                raise KeyError(id)
            return default

    def set(self, id, value):
        if not interfaces.IMutableCollection.providedBy(self):
            raise NotImplementedError
        if not self.type.providedBy(value):
            raise TypeError(value)
        try:
            self.__components[self.__ids.index(id)] = value
        except ValueError:
            raise KeyError(id)

    def append(self, component):
        if self.type.providedBy(component):
            if component.identifier not in self.__ids:
                self.__ids.append(component.identifier)
                self.__components.append(component)
            else:
                raise ValueError(
                    u"Duplicate identifier", component.identifier)
        else:
            raise TypeError(u"Invalid type", component)

    def extend(self, *components):
        for cmp in components:
            if self.type.providedBy(cmp):
                self.append(cmp)
            elif interfaces.IIterable.providedBy(cmp):
                for item in cmp:
                    self.append(item)
            else:
                if self.factory is not None:
                    loadComponents()
                    factory = component.queryAdapter(cmp, self.factory)
                    if factory is not None:
                        for item in factory.produce():
                            self.append(item)
                        continue
                raise TypeError(u'Invalid type', cmp)

    def select(self, *ids):
        components = (c for c in self.__components if c.identifier in ids)
        return self.__class__(*components, **self.__options)

    def omit(self, *ids):
        components = (c for c in self.__components if c.identifier not in ids)
        return self.__class__(*components, **self.__options)

    def copy(self):
        return self.__class__(*self.__components, **self.__options)

    def keys(self):
        return list(self.__ids)

    def __add__(self, other):
        if interfaces.ICollection.providedBy(other):
            copy = self.copy()
            for component in other:
                copy.append(component)
            return copy
        if interfaces.IComponent.providedBy(other):
            copy = self.copy()
            copy.append(other)
            return copy
        raise NotImplementedError

    def __getitem__(self, id):
        return self.get(id)

    def __setitem__(self, id, value):
        self.set(id, value)

    def __delitem__(self, id):
        if not interfaces.IMutableCollection.providedBy(self):
            raise NotImplementedError
        try:
            index = self.__ids.index(id)
            self.__ids.remove(id)
            del self.__components[index]
        except ValueError:
            raise KeyError(id)

    def __contains__(self, id):
        return id in self.__ids

    def __iter__(self):
        return self.__components.__iter__()

    def __len__(self):
        return self.__components.__len__()

    def __repr__(self):
        return "<%s>" % (self.__class__.__name__)




from zeam.form.base.actions import Action, Actions, action, CompoundActions
from zeam.form.base.fields import Field, Fields
from zeam.form.base.form import Form, FormData, extends
from zeam.form.base.interfaces import ActionError
from zeam.form.base.markers import FAILURE, SUCCESS, NOTHING_DONE
from zeam.form.base.markers import NO_VALUE, NO_CHANGE
from zeam.form.base.markers import DISPLAY, INPUT, DEFAULT, HIDDEN
from zeam.form.base.widgets import Widgets
from zeam.form.base.datamanager import ObjectDataManager, DictDataManager
from zeam.form.base.datamanager import NoneDataManager
from zeam.form.base.datamanager import makeAdaptiveDataManager
from zeam.form.base.sorters import sort_components

from grokcore.component import context, name
from grokcore.view import layer, template, require

from zeam.form.base.interfaces import IZeamFormBaseAPI
__all__ = list(IZeamFormBaseAPI)



"""
We are going to define a custom widget which doesn't provides its own
to render himself:

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.widgets.widgetnorenderortemplate')
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: <class 'martian.error.GrokError'>: Widget <class 'zeam.form.base.ftests.widgets.widgetnorenderortemplate.MyWidget'> has no associated template or 'render' method.
    in:

"""

from zeam.form.base.fields import Field
from zeam.form.base.widgets import Widget
from zeam.form.base import interfaces

from zope.interface import Interface

from grokcore import component as grok


class MyField(Field):
    """A custom field.
    """


class MyWidget(Widget):
    """Custom widget to render my field
    """
    grok.adapts(MyField, interfaces.IFormCanvas, Interface)



"""
We define here a simple form with two fields and one action registered
with a decorator.

Let's grok our example:

  >>> from zeam.form.base.testing import grok
  >>> grok('zeam.form.base.ftests.forms.inputform')

We can now lookup our form by the name of its class:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.base.ftests.forms.inputform import Context
  >>> context = Context()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='registration')
  >>> form
  <zeam.form.base.ftests.forms.inputform.Registration object at ...>

  >>> len(form.fields)
  2
  >>> len(form.actions)
  1

Integration tests
-----------------

  >>> root = getRootFolder()
  >>> root['content'] = context

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

Empty submission
~~~~~~~~~~~~~~~~

We are going just to submit the form without giving any required
information, and we should get an error:

  >>> browser.open('http://localhost/content/registration')
  >>> action = browser.getControl('Register')
  >>> action
  <SubmitControl name='form.action.register' type='submit'>

  >>> action.click()

  >>> 'Missing required value' in browser.contents
  True
  >>> 'Registered' in browser.contents
  False

Valid submission
~~~~~~~~~~~~~~~~

Let's get our control for fields and filled them, and submit the form:

  >>> browser.open('http://localhost/content/registration')
  >>> name = browser.getControl('Name')
  >>> name
  <Control name='form.field.name' type='text'>
  >>> name.value = 'Sylvain Viollon'
  >>> job = browser.getControl('Job')
  >>> job
  <Control name='form.field.job' type='text'>
  >>> job.value = 'Developer'

  >>> browser.getControl('Register').click()

  >>> 'Registered Sylvain Viollon as Developer' in browser.contents
  True

Our action says that you can ignore the request if it succeed (and it
is the case here):

  >>> browser.getControl('Name').value
  ''
  >>> browser.getControl('Job').value
  ''

Incomplete submission
~~~~~~~~~~~~~~~~~~~~~

In case of an incomplete submission, fields should keep the value they
got for that submission:

  >>> browser.open('http://localhost/content/registration')
  >>> job = browser.getControl('Job')
  >>> job
  <Control name='form.field.job' type='text'>
  >>> job.value = 'Designer'

  >>> browser.getControl('Register').click()

  >>> 'Missing required value' in browser.contents
  True
  >>> 'Registered' in browser.contents
  False

  >>> new_job = browser.getControl('Job')
  >>> new_job.value
  'Designer'

And so now we can finish to submit the form, and form values should be
gone (as we successfully submit the form):

  >>> browser.getControl('Name').value = 'Wim Boucqueart'
  >>> browser.getControl('Register').click()

  >>> 'Missing required value' in browser.contents
  False
  >>> 'Registered Wim Boucqueart as Designer' in browser.contents
  True

  >>> browser.getControl('Name').value
  ''
  >>> browser.getControl('Job').value
  ''


"""


from zeam.form import base
from grokcore import component as grok

class Context(grok.Context):
    pass


class Registration(base.Form):

    label = u"My form"
    description = u"The description of my form"
    fields = base.Fields(base.Field("Name"), base.Field("Job"))
    fields['name'].description = 'Name of the candidate'
    fields['name'].required = True

    @base.action(u"Register")
    def register(self):
        data, errors = self.extractData()
        if errors:
            return
        # In case of success we don't keep request value in the form
        self.ignoreRequest = True
        self.status = u"Registered %(name)s as %(job)s" % data



# -*- coding: utf-8 -*-

from zope import interface
from zope.publisher.interfaces.browser import IBrowserPage


class IModeMarker(interface.Interface):
    """This interface identifies a form mode and defines if it allows
    data extraction.
    """
    extractable = interface.Attribute(
        u"Boolean allowing or not the extraction of the data,"
        u" for components in that mode.")


class IComponent(interface.Interface):
    """A named component.
    """
    identifier = interface.Attribute(u"Component id")
    title = interface.Attribute(u"Component title")

    def clone(new_identifier=None):
        """Return a clone of the new component, with identifier
        new_identifier if it is not None.
        """


class IComponentFactory(interface.Interface):
    """Component used to built components.
    """

    def produce(self):
        """Should generate components.
        """


class IIterable(interface.Interface):
    """An iterable that return a component upon each iteration.
    """

    def __iter__():
        """Return an iterator on the components.
        """


class ICollection(IIterable):
    """Support to manage a collection of ordered named components.
    """
    type = interface.Attribute(
        u"Interface restricting the type of component")
    factory = interface.Attribute(
        u"Interface to query in order to get a factory to extend "
        u"collection with unknow components.")

    def append(component):
        """Add a new component to the collection. Modify the current
        collection.
        """

    def extend(*component):
        """Create/Add a list of components to the collection. Modify
        the current collection.
        """

    def get(id, default=None):
        """Return the component with the given ID.
        """

    def select(*ids):
        """Return a copy containing only the given named components.
        """

    def omit(*ids):
        """Return a copy containing all but none of the given named
        components.
        """

    def copy():
        """Return a copy of the collection.
        """

    def clear():
        """Empty the collection: remove all components from it.
        """

    def keys():
        """Return all components id contained in the collection.
        """

    def __add__(other):
        """Create a collection as copy of self, and add value for
        other component or collection.
        """

    def __getitem__(id):
        """Return the given component identified by id or raise
        KeyError.
        """

    def __contains__(id):
        """Return true if the collection contains a component
        identified by id.
        """

    def __len__():
        """Return the numbre of components.
        """


class IMutableCollection(ICollection):
    """A collection that can be changed.
    """

    def set(id , value):
        """Change component associated to this id.
        """

    def __setitem__(id, value):
        """Change component associated to this id.
        """

    def __delitem__(id):
        """Remove the component associated to this id.
        """


class IPrefixable(interface.Interface):
    """An object with a prefix.
    """
    prefix = interface.Attribute("Prefix")


class IRenderableComponent(IPrefixable, IComponent):
    """A component that can be rendered with the help of a widget.
    """
    mode = interface.Attribute(
        u"Mode should be used to render the component")

    def available(form):
        """Return a boolean to qualify if the component wants to be
        rendered in the given context (i.e. form).
        """


class IFieldExtractionValueSetting(IPrefixable):
    """Setting to extract field values.
    """
    ignoreRequest = interface.Attribute(u"Ignore request values")
    ignoreContent = interface.Attribute(u"Ignore content values")
    mode = interface.Attribute(
        u"Mode should be used to render all the widgets")


class IDataManager(interface.Interface):
    """A data manager let you access content.
    """

    def __init__(content):
        """Initialize the data manager for the given content.
        """

    def getContent():
        """Return the content managed by this data manager.
        """

    def get(identifier):
        """Return content value associated to the given identifier or
        raise KeyError.
        """

    def set(identifier, value):
        """Modifiy content value associated to the given identifier.
        """


class IFormData(IFieldExtractionValueSetting):
    """Form data processing facilities.
    """
    dataManager = interface.Attribute(
        u"Data manager class used to access content.")
    status = interface.Attribute(
        u"Form status message.")
    errors = interface.Attribute(
        u"Iterable of the errors that occured during the form processing.")
    formErrors = interface.Attribute(
        u"Main errors that occurred during the form processing.")

    def getContentData():
        """Returns the content that will be used for the form processing.
        """

    def setContentData(content):
        """Sets the content that will be used as the form processing context.
        """

    def validateData(fields, data, errors):
        """Validates the form in a global way and returns a collection
        of errors (if any occured) or None.
        """

    def extractData(fields):
        """Returns the form data and errors for the given fields.
        """


class ActionError(Exception):
    """A error happening while processing the form.
    """


class IAction(IRenderableComponent):
    """A form action.
    """
    description = interface.Attribute(u"Describe the action")

    def validate(form):
        """Self validation of values in order to run.
        """

    def __call__(form):
        """Execute the action.
        """


class IActions(ICollection):
    """A list of actions.
    """

    def process(form, request):
        """Execute actions.
        """


class IField(IRenderableComponent, IFieldExtractionValueSetting):
    """A form field.
    """
    description = interface.Attribute(u"Field description")
    required = interface.Attribute(
        u"Boolean indicating if the field is required")
    readonly = interface.Attribute(
        u"Boolean indicating if the field is read-only")

    def getDefaultValue(form):
        """Return the default value.
        """

    def validate(value, form):
        """Validate that the given value fullfil the field
        requirement.
        """


class IFieldFactory(IComponentFactory):
    """Factory to create zeam.form Fields from other components than
    zeam.form ones.
    """


class IFields(ICollection):
    """A collection of fields.
    """


class IError(IComponent):
    """A error.
    """

    def get(prefix, default=None):
        """Return a sub error identified by the given prefix if
        available.
        """


class IErrors(IMutableCollection):
    """A collection of errors.
    """


class IWidget(IComponent):
    """Display a form component on the page.
    """

    def htmlId():
        """Return the HTML id of the HTML component representing the
        widget.
        """

    def htmlClass():
        """Return an HTML class to mark the widget with.
        """

    def render():
        """Return the rendered HTML of the widget.
        """


class IFieldWidget(IWidget):
    """Widget for fields.
    """
    description = interface.Attribute(u"Description of the field")
    error = interface.Attribute(u"Field error, or None")
    required = interface.Attribute(
        u"Boolean indicating if the field is required")
    readonly = interface.Attribute(u"Boolean indicating if field is read-only")


class IWidgetExtractor(interface.Interface):
    """The counterpart of the Widget, used to extract widget value
    from the request.
    """

    def extract():
        """Return a tuple (value, error). Value must be a valid field
        value. If error is not None, value is discarded.
        """

    def extractRaw():
        """Return request entries needed for the widget to redisplay
        the same information in case of validation failure.
        """


class IWidgets(ICollection):
    """A collection of widgets.
    """


class IGrokViewSupport(interface.Interface):
    """Some usefull methods from Grok View.
    """
    response = interface.Attribute(
        u"Response object that is associated with the current request.")
    i18nLanguage = interface.Attribute(
        u"Language code to translate view content to.")

    def redirect(url):
        """Redirect to given URL.
        """

    def url(obj=None, name=None, data=None):
        """Compute an URL to an object (or self), and add a name, and
        maybe some data.
        """

    def update():
        """User defined pre-update.
        """

    def render():
        """Render the form.
        """


class IFormCanvas(IPrefixable, IFieldExtractionValueSetting, IGrokViewSupport):
    """Definition of a form structure.
    Form presentation : label, description
    Form contents and actions : fields, actions and their related methods.
    """
    label = interface.Attribute(u"Form title")
    description = interface.Attribute(u"Form description")

    actions = interface.Attribute(u"Form actions")
    fields = interface.Attribute(u"Form fields")

    def htmlId():
        """Return an identifier that can be used in the HTML code to
        identify the form.
        """

    def haveRequiredFields():
        """Return an boolean True if any of the fields are required.
        """

    def updateActions():
        """Set up and run form actions.
        """

    def updateWidgets():
        """Set up rendering field / action widgets and their value to
        display.
        """


class ISimpleFormCanvas(IFormCanvas, IFormData):
    """A simple form canvas with only fields and actions.
    """
    actionWidgets = interface.Attribute(u"Form widgets")
    fieldWidgets = interface.Attribute(u"Form widgets")


class IForm(IBrowserPage, IFormCanvas):
    """Regular form containing fields and actions, that you can call,
    and will be updated and rendered.
    """

    def updateForm():
        """Update the form mechanism (setup fields, actions, run
        actions, setup widgets).
        """

    def __call__():
        """Update and render the form.
        """


class ISimpleForm(IForm, ISimpleFormCanvas):
    """A simple form, with fields and actions.
    """


class IZeamFormBaseAPI(interface.Interface):
    """Base zeam form API.
    """
    Action = interface.Attribute(
        u"A form action")
    Actions = interface.Attribute(
        u"A collection of actions")
    CompoundActions = interface.Attribute(
        u"A collection of actions that can be managed differently")
    Field = interface.Attribute(
        u"A form field")
    Fields = interface.Attribute(
        u"A collection of fields")
    Widgets = interface.Attribute(
        u"A collection of widgets")
    FormData = interface.Attribute(
        u"A configuration object to render fields as widgets")
    Form = interface.Attribute(
        u"A basic and simple Form")
    ActionError = interface.Attribute(
        u"An error occuring while processing an Action")

    action = interface.Attribute(
        u"Decorator to use a form method as an Action")

    ObjectDataManager = interface.Attribute(
        u"Data manager to work with values as attribute of an object")
    DictDataManager = interface.Attribute(
        u"Data manager to work with values in dictionary")
    NoneDataManager = interface.Attribute(
        u"Data manager to work directly with a value")
    makeAdaptiveDataManager = interface.Attribute(
        u"Data manager to work with from an simple adapter")

    context = interface.Attribute(
        u"Directive to map form to a context")
    name = interface.Attribute(
        u"Directive to name a form")
    layer = interface.Attribute(
        u"Directive to specify a form layer")
    template = interface.Attribute(
        u"Directive to specify a grok template to render the form")
    require = interface.Attribute(
        u"Directive to require a permission to access the form")

    NO_VALUE = interface.Attribute(
        u"Marker to mark the absence of a value")
    NO_CHANGE = interface.Attribute(
        u"Marker to mark the non-modification of a value")
    DISPLAY = interface.Attribute(
        u"Marker for mode to get display widgets")
    INPUT = interface.Attribute(
        u"Marker for mode to get input widgets")
    HIDDEN = interface.Attribute(
        u"Marker for mode to get hidden widgets")
    DEFAULT = interface.Attribute(
        u"Marker used to use the default value located on the form")
    SUCCESS = interface.Attribute(
        u"Marker used by actions to report a success")
    FAILURE = interface.Attribute(
        u"Marker used by actions to report a failure")
    NOTHING_DONE = interface.Attribute(
        u"Marker used by actions to report the absence of activity")
